## 5-Vorzeichen-Tiefe (bereits normierte Version fuer den Test)

# input: residuals (Residuen des Modells unter einem Parameter)
# output: normierte Version der 5-Vorzeichen-Tiefe

linear_5depth <- function(residuals){
  N <- length(residuals)
  S <- cumsum(sign(residuals))
  S2 <- cumsum(S)
  S3 <-  cumsum(S * ((1:N)^2 -(1:N - 1)^2))
  
  # Hilfsfunktion
  prod_one_factor <- function(residuals = residuals, K = 4, L){
    s <- sign(residuals)
    N <- length(residuals)
    M_L <- length(which(s[1:L-1] == -1))
    M_R <- length(which(s[L:N] == -1))
    ind <- 0
    temp1 <- 0
    for(j in L:(N-K+L)){
      if(s[j] == -1){
        ind <- 1
        M_R <- M_R - 1
        temp3 <- 1}else{
          ind <- 0
          temp3 <- 0}
      temp2 <- 0
      for(k_1 in 0:(L-1)){
        for(k_2 in 0:(K-L)){
          temp2 <- temp2 + (-1)^(k_1+k_2) * choose(M_L, k_1) * choose(j - M_L - 1, L - 1 - k_1) * 
            choose(M_R, k_2) * choose(N - j - M_R, K - L  - k_2) * -(2*temp3-1)
        }
      }
      temp1 <- temp1 + j * temp2
      M_L <- M_L + temp3
    }
    return(temp1)
  }
  
  # Hilfsfunktion
  linearprod <- function(residuals = residuals, K = 4){
    M <- length(which(sign(residuals) == -1))
    N <- length(residuals)
    C <- 0
    for(i in 0:K){
      C <- C + choose(M,i) * choose(N-M,K-i) * (-1)^i
    }
    return(C)
  }
  
  
  return(5 * N^4/((N-1)*(N-2)*(N-3)*(N-4)) * (1/8 + 1/8 * 1/N * S[N]^2 
                                              - 3/4 * 1/N^2 * (sum(S^2) + sum((S[N] - S)^2))
                                              + 3/2 * 1/N^3 * (sum((2*(N - 1:N) + 1) * S^2) + 
                                                                 sum(((2 * 1:N) - 1) * (S[N] - S)^2) +
                                                                 2 * N * sum(S^2) - 2 * sum(S)^2)
                                              - 1/N^4 * (sum(((N - 1:N + 1)^3 - (N - 1:N)^3) * S^2)  + 
                                                           3 *( sum((1:N) * ((N - 1:N + 1)^2 - (N - 1:N)^2) * S^2) 
                                                                + sum((N - 1:N + 1)^2 * S^2)
                                                                - 2 * sum( ((N - 1:N + 1)^2 - (N - 1:N)^2) * S * S2 )) +
                                                           3 *  (sum((1:N)^2 * S^2) 
                                                                 + sum((N - 1:N + 1) * ((1:N)^2 - (1:N - 1)^2) * S^2)
                                                                 - 2 * sum(S * S3))  +
                                                           sum(((1:N)^3 - (1:N - 1)^3) * (S[N] - S)^2))) 
         + N/(16 * choose(N,5)) * (N * linearprod(residuals, 4)
                                   -2 * prod_one_factor(residuals, 4, 4)
                                   +2 * prod_one_factor(residuals, 4, 3)
                                   -2 * prod_one_factor(residuals, 4, 2)
                                   +2 * prod_one_factor(residuals, 4, 1))
         +
           N^4/(32 * choose(N,5)) * (+(1/(2*N)*(S[N]^2 - N)
                                       - 4/(3*N^2)*(S[N]^2 - N)
                                       -1/N * S[N]^2 + 1/N^2 * (sum(S^2) + sum((S[N] - S)^2))
                                       + 8/(3*N^2) * S[N]^2 - 8/(3*N^3) * (sum(S^2) + sum((S[N] - S)^2)) )))
  return(result)
}




###############################################################################
## Beispiel: quadratische Regression
set.seed(9719)
x <- seq(-2, 2, 0.25) # Regressoren
N <- length(x)
y <- x^2 + 2 + rnorm(N) # Modell: x^2 + 2
res <- y - (-x^2 + 3) # vorgeschlagenes Modell: - x^2 + 4

linear_5depth(res) # normierter Wert (fuer den Test zu verwenden)
linear_5depth(res)/N + 1/16 # Definition der Tiefe aus Uebungsblatt 14

