## Asymptotische Version der K-Vorzeichen-Tiefe in linearer Zeit
## (bereits normierte Version fuer den Test)

# input:  residuals (Residuen des Modells unter einem Parameter)
#         K (Parameter bei der K-Vorzeichen-Tiefe)
#         approx_lower_factorial (binre Variable, die die Folge im Vorfaktor
#         approximiert, sinnvoll fr N gro)
# output: normierte Version der K-Vorzeichen-Tiefe

asymp_K_depth <- function(res, K, approx_lower_factorial = FALSE){
  res <- sign(res)
  N <- length(res)
  S <- matrix(0, nrow = K-1, ncol=N)
  
  for(j in 1:(K-1)){
    S[j, ] <- cumsum((0.5-(1:N)/N)^(j-1)*res)
  }
  
  depth <- 0
  for(j in 0:(K-2)){
    depth <- depth + choose(K-2,j)*sum( ((1:N)/N)^j*res*(S[K-1-j,N] 
                                                             - S[K-1-j, ]))
  }
  if(approx_lower_factorial){
    scaling <- -K*(K-1)/(2*N)
  }
  else{
    scaling <- -K*(K-1)*N^(K-1)/(2*prod(seq(N-K+1,N,by=1)))
  }
  return(scaling*depth)
}


### Beispiele

## Berechne K-Tiefe fr K = 3, ..., 10
res <- rep(c(rep(1, 5), rep(-1, 5)), 4)
asymp_K_depth(res, 3)
asymp_K_depth(res, 4)
asymp_K_depth(res, 5)
asymp_K_depth(res, 6)
asymp_K_depth(res, 7)
asymp_K_depth(res, 8)
asymp_K_depth(res, 9)
asymp_K_depth(res, 10)

## checke, ob 5%-Quantil der 5-Vorzeichentiefe passt
mean(replicate(1000, asymp_K_depth(rnorm(1000), 5) < -0.6717046))