
############### asymptotic implementation used ###################
##################################################################
asymp_K_depth <- function(res, K, approx_lower_factorial=FALSE){
  sgn_res <- sign(res)
  N <- length(sgn_res)
  
  S <- matrix(0, nrow = K-1, ncol=N)
  
  for(j in 1:(K-1)){
    S[j, ] <- cumsum((0.5-(1:N)/N)^(j-1)*sgn_res)
  }
  
  depth <- 0
  for(j in 0:(K-2)){
    depth <- depth + choose(K-2,j)*sum( ((1:N)/N)^j*sgn_res*(S[K-1-j,N] - S[K-1-j, ]))
  }
  if(approx_lower_factorial){
    scaling <- -K*(K-1)/(2*N)
  }
  else{
    scaling <- -K*(K-1)*N^(K-1)/(2*prod(seq(N-K+1,N,by=1)))
  }
  return(scaling*depth)
}

###############################################################################################
################ some helpful functions #######################################################

generate_blocks <- function(N,M){
  x <- numeric(0)
  if(N%%M == 0){
    for(i in 1:M){
      x <- c(x, rep((-1)^i, ceiling(N/M)))
    }
  }else{
    N_temp <- N-(N%%M)
    for(i in 1:M){
      x <- c(x, rep((-1)^i, N_temp/M))
    }
    x <- c(x, rep((-1)^M, N%%M))
  }
  return(x)}

generator_extra <- function(N){
  return(c(rep(1,N/6), rep(-1, N/3), rep(1, N/3), rep(-1, N/6)))
}

generator_extra35 <- function(N){
  return(c(rep(1, N/5), rep(-1, N/5), rep(1, N/5), rep(-1, N/5), rep(1, N/5)))
}

generator_extra4 <- function(N){
  return(c(rep(1, 1), rep(-1, N/4), rep(1, N/4), rep(-1, N/4), rep(1, N-(3/4*N+1))))
}

###############################################################################################
############## TWO and THREE sign changes #####################################################
###############################################################################################

### generate ecdfs ############################################
set.seed(2409)
N <- seq(12, 160, 12)

quants_3depth <- list(replicate(100000, asymp_K_depth(rnorm(12),3)))
quants_4depth <- list(replicate(100000, asymp_K_depth(rnorm(12),4)))
quants_5depth <- list(replicate(100000, asymp_K_depth(rnorm(12),5)))

for(n in N[2:length(N)]){
  quants_3depth <- c(quants_3depth, list(replicate(100000, asymp_K_depth(rnorm(n),3))))
  quants_4depth <- c(quants_4depth, list(replicate(100000, asymp_K_depth(rnorm(n),4))))
  quants_5depth <- c(quants_5depth, list(replicate(100000, asymp_K_depth(rnorm(n),5))))
}
#################################################################

### generate ecdfs ############################################

set.seed(2409)
N1 <- seq(10, 160, 10)
N2 <- N

quants_3depthZEHNER <- list(replicate(100000, asymp_K_depth(rnorm(10),3)))
quants_5depthZEHNER <- list(replicate(100000, asymp_K_depth(rnorm(10),5)))

for(n in N1[2:length(N1)]){
  quants_3depthZEHNER <- c(quants_3depthZEHNER, list(replicate(100000, asymp_K_depth(rnorm(n),3))))
  quants_5depthZEHNER <- c(quants_5depthZEHNER, list(replicate(100000, asymp_K_depth(rnorm(n),5))))
  
}

### compute p-values

### two sign changes

two_sign_3depth <- sapply(N, function(N){asymp_K_depth(generate_blocks(N, 3),3)})
two_sign_4depth <- sapply(N, function(N){asymp_K_depth(generate_blocks(N, 3),4)})
two_sign_5depth <- sapply(N, function(N){asymp_K_depth(generate_blocks(N, 3),5)})

### three sign changes

three_sign_3depth <- sapply(N, function(N){asymp_K_depth(generator_extra(N),3)})
three_sign_4depth <- sapply(N, function(N){asymp_K_depth(generate_blocks(N, 4),4)})
three_sign_5depth <- sapply(N, function(N){asymp_K_depth(generate_blocks(N, 4),5)})

################################################################################

# p-value plots (7.3 x 4 inches)

plot(N, sapply(1:length(N), function(n){length(which(two_sign_3depth[n] > quants_3depth[[n]]))/1e5}),
     type = "o", ylab = "p-value", xlab = "N", main = "Two sign changes", pch = 17,
     cex.axis = 1.8, cex.main = 1.8, cex.lab = 1.8, ylim = c(0, 0.8), xlim = c(0,160), lwd = 2)
lines(N, sapply(1:length(N), function(n){length(which(two_sign_4depth[n] > quants_4depth[[n]]))/1e5}),
      col = "red", type = "o", pch = 15, lwd = 2)
legend("topright", legend = c("3-depth", "4-depth"), col = c("black", "red"),
       pch = c(17,15), cex = 1.3, pt.cex = 1, lwd = c(2,2))

plot(N, sapply(1:length(N), function(n){length(which(three_sign_3depth[n] > quants_3depth[[n]]))/1e5}),
type = "o", ylab = "p-value", xlab = "N", main = "Three sign changes", pch = 17,
cex.axis = 1.8, cex.main = 1.8, cex.lab = 1.8, ylim = c(0, 0.8), xlim = c(0,160), lwd = 2)
lines(N,sapply(1:length(N), function(n){length(which(three_sign_4depth[n] > quants_4depth[[n]]))/1e5}),
col = "red", type = "o", pch = 15, lwd = 2)
legend("topright", legend = c("3-depth", "4-depth"), col = c("black", "red"),
       pch = c(17,15), cex = 1.3, pt.cex = 1, lwd = c(2,2))

## 5-depth with three sign changes:
plot(N, sapply(1:length(N), function(n){length(which(three_sign_3depth[n] > quants_3depth[[n]]))/1e5}),
     type = "o", ylab = "p-value", xlab = "N", main = "Three sign changes", pch = 17,
     cex.axis = 1.8, cex.main = 1.8, cex.lab = 1.8, ylim = c(0, 0.8), xlim = c(0,160), lwd = 2)
lines(N,sapply(1:length(N), function(n){length(which(three_sign_5depth[n] > quants_5depth[[n]]))/1e5}),
      col = "darkgreen", type = "o", pch = 8, lwd = 2)
lines(N,sapply(1:length(N), function(n){length(which(three_sign_4depth[n] > quants_4depth[[n]]))/1e5}),
      col = "red", type = "o", pch = 15, lwd = 2)
legend("topright", legend = c("3-depth", "4-depth", "5-depth"), col = c("black", "red", "darkgreen"),
       pch = c(17,15,8), cex = 1.3, pt.cex = 1, lwd = c(2,2,2))

###############################################################################################
############## FOUR sign changes ##############################################################
###############################################################################################

### compute p-values

### four sign changes

four_sign_3depth <- sapply(N1, function(N){asymp_K_depth(generator_extra35(N),3)})
four_sign_4depth <- sapply(N2, function(N){asymp_K_depth(generator_extra4(N),4)})
four_sign_5depth <- sapply(N1, function(N){asymp_K_depth(generator_extra35(N),5)})

################################################################################

# p-value plots

plot(N1, sapply(1:length(N1), function(n){length(which(four_sign_3depth[n] > quants_3depthZEHNER[[n]]))/1e5}),
     type = "o", ylab = "p-value", xlab = "N", main = "Four sign changes", pch = 17,
     cex.axis = 1.8, cex.main = 1.8, cex.lab = 1.8, ylim = c(0, 0.95), lwd = 2, xlim = c(0,160))
lines(N1, sapply(1:length(N1), function(n){length(which(four_sign_5depth[n] > quants_5depthZEHNER[[n]]))/1e5}),
      col = "darkgreen", type = "o", pch = 8, lwd = 2)
lines(N2, sapply(1:length(N2), function(n){length(which(four_sign_4depth[n] > quants_4depth[[n]]))/1e5}),
      col = "red", type = "o", pch = 15, lwd = 2)
legend("topright", legend = c("3-depth", "4-depth", "5-depth"), 
       col = c("black", "red", "darkgreen"), pch = c(17,15, 8), cex = 1.3, pt.cex = 1.3, lwd = c(2,2,2))

plot(N1, sapply(1:length(N1), function(n){length(which(four_sign_3depth[n] > quants_3depthZEHNER[[n]]))/1e5}),
     type = "o", ylab = "p-value", xlab = "N", main = "Four sign changes", pch = 17,
     cex.axis = 1.8, cex.main = 1.8, cex.lab = 1.8, ylim = c(0, 0.8), lwd = 2, xlim = c(0,160))
lines(N1, sapply(1:length(N1), function(n){length(which(four_sign_5depth[n] > quants_5depthZEHNER[[n]]))/1e5}),
      col = "darkgreen", type = "o", pch = 8, lwd = 2)
lines(N2, sapply(1:length(N2), function(n){length(which(four_sign_4depth[n] > quants_4depth[[n]]))/1e5}),
      col = "red", type = "o", pch = 15, lwd = 2)
legend("topright", legend = c("3-depth", "4-depth", "5-depth"), 
       col = c("black", "red", "darkgreen"), pch = c(17,15, 8), cex = 1.3, pt.cex = 1.3, lwd = c(2,2,2))


plot(N1, sapply(1:length(N1), function(n){length(which(four_sign_3depth[n] > quants_3depthZEHNER[[n]]))/1e5}),
     type = "o", ylab = "p-value", xlab = "N", main = "Four sign changes", pch = 17,
     cex.axis = 1.8, cex.main = 1.8, cex.lab = 1.8, ylim = c(0, 0.95), lwd = 2, xlim = c(0,160))
lines(N2, sapply(1:length(N2), function(n){length(which(four_sign_4depth[n] > quants_4depth[[n]]))/1e5}),
      col = "red", type = "o", pch = 15, lwd = 2)
legend("topright", legend = c("3-depth", "4-depth"), 
       col = c("black", "red"), pch = c(17,15), cex = 1.3, pt.cex = 1, lwd = c(2,2))

plot(N1, sapply(1:length(N1), function(n){length(which(four_sign_3depth[n] > quants_3depthZEHNER[[n]]))/1e5}),
     type = "o", ylab = "p-value", xlab = "N", main = "Four sign changes", pch = 17,
     cex.axis = 1.8, cex.main = 1.8, cex.lab = 1.8, ylim = c(0, 0.8), lwd = 2, xlim = c(0,160))
lines(N2, sapply(1:length(N2), function(n){length(which(four_sign_4depth[n] > quants_4depth[[n]]))/1e5}),
      col = "red", type = "o", pch = 15, lwd = 2)
legend("topright", legend = c("3-depth", "4-depth"), 
       col = c("black", "red"), pch = c(17,15), cex = 1.3, pt.cex = 1, lwd = c(2,2))

###############################################################################################
############## FIVE sign changes ##############################################################
##############################################################################################

generator_extra35_5 <- function(N){
  return(c(rep(1, N/10), rep(-1, N/5), rep(1, N/5), rep(-1, N/5), rep(1, N/5), rep(-1, N/10)))
}

generator_extra46_5 <- function(N){
  return(c(rep(1, N/6), rep(-1, N/6), rep(1, N/6), rep(-1, N/6), rep(1, N/6), rep(-1, N/6)))
}

### generate ecdfs ############################################

set.seed(2409)
N1 <- seq(10, 160, 10)
N2 <- seq(12, 160, 12)

quants_6depth <- list(replicate(100000, asymp_K_depth(rnorm(12),6)))

for(n in N2[2:length(N2)]){
  quants_6depth <- c(quants_6depth, list(replicate(100000, asymp_K_depth(rnorm(n),6))))
}

#################################################################
### compute p-values

### five sign changes

five_sign_3depth <- sapply(N1, function(N){asymp_K_depth(generator_extra35_5(N),3)})
five_sign_4depth <- sapply(N2, function(N){asymp_K_depth(generator_extra46_5(N),4)})
five_sign_5depth <- sapply(N1, function(N){asymp_K_depth(generator_extra35_5(N),5)})
five_sign_6depth <- sapply(N2, function(N){asymp_K_depth(generator_extra46_5(N),6)})

################################################################################

# p-value plots

plot(N1, sapply(1:length(N1), function(n){length(which(five_sign_3depth[n] > quants_3depthZEHNER[[n]]))/1e5}),
     type = "o", ylab = "p-value", xlab = "N", main = "Five sign changes", pch = 17, ylim = c(0,1), lwd = 2,
     cex.axis = 1.5, cex.main = 1.5, cex.lab = 1.5)
lines(N2, sapply(1:length(N2), function(n){length(which(five_sign_4depth[n] > quants_4depth[[n]]))/1e5}),
      col = "red", type = "o", pch = 15, lwd = 2)
lines(N1, sapply(1:length(N1), function(n){length(which(five_sign_5depth[n] > quants_5depthZEHNER[[n]]))/1e5}),
      col = "darkgreen", type = "o", pch = 8, lwd = 2)
lines(N2, sapply(1:length(N2), function(n){length(which(five_sign_6depth[n] > quants_6depth[[n]]))/1e5}),
      col = "brown", type = "o", pch = 10, lwd = 2)
legend("topright", legend = c("3-depth", "4-depth", "5-depth","6-depth"), 
       col = c("black", "red", "darkgreen","brown"), pch = c(17,15, 8,10), lwd = c(2,2,2,2), cex = 1.5)


###############################################################################################
############## SIX sign changes ##############################################################
##############################################################################################

generator_extra357 <- function(N){
  return(c(rep(1, N/7), rep(-1, N/7), rep(1, N/7), rep(-1, N/7), rep(1, N/7), rep(-1, N/7), rep(1, N/7)))
}

generator_extra46 <- function(N){
  return(c(rep(1, N/12), rep(-1, N/6), rep(1, N/6), rep(-1, N/6), rep(1, N/6), rep(-1, N/6), rep(1, N/12)))
}

### generate ecdfs ############################################

set.seed(2409)
N1 <- seq(7, 84, 7)
N2 <- seq(12, 84, 12)

quants_3depth <- list(replicate(1000, asymp_K_depth(rnorm(7),3)))
quants_4depth <- list(replicate(1000, asymp_K_depth(rnorm(12),4)))
quants_5depth <- list(replicate(1000, asymp_K_depth(rnorm(7),5)))
quants_6depth <- list(replicate(1000, asymp_K_depth(rnorm(12),6)))
quants_7depth <- list(replicate(1000, asymp_K_depth(rnorm(7),7)))

for(n in N1[2:length(N1)]){
  quants_3depth <- c(quants_3depth, list(replicate(1000, asymp_K_depth(rnorm(n),3))))
  quants_5depth <- c(quants_5depth, list(replicate(1000, asymp_K_depth(rnorm(n),5))))
  quants_7depth <- c(quants_7depth, list(replicate(1000, asymp_K_depth(rnorm(n),7))))
}

for(n in N2[2:length(N2)]){
  quants_4depth <- c(quants_4depth, list(replicate(1000, asymp_K_depth(rnorm(n),4))))
  quants_6depth <- c(quants_6depth, list(replicate(1000, asymp_K_depth(rnorm(n),6))))
}

#################################################################
### compute p-values

### six sign changes

six_sign_3depth <- sapply(N1, function(N){asymp_K_depth(generator_extra357(N),3)})
six_sign_4depth <- sapply(N2, function(N){asymp_K_depth(generator_extra46(N),4)})
six_sign_5depth <- sapply(N1, function(N){asymp_K_depth(generator_extra357(N),5)})
six_sign_6depth <- sapply(N2, function(N){asymp_K_depth(generator_extra46(N),6)})
six_sign_7depth <- sapply(N1, function(N){asymp_K_depth(generator_extra357(N),7)})

################################################################################

# p-value plots

plot(N1, sapply(1:length(N1), function(n){length(which(six_sign_3depth[n] > quants_3depth[[n]]))/1e3}),
     type = "o", ylab = "p-value", xlab = "N", main = "Six sign changes", pch = 17, ylim = c(0,1))
lines(N2, sapply(1:length(N2), function(n){length(which(six_sign_4depth[n] > quants_4depth[[n]]))/1e3}),
      col = "red", type = "o", pch = 15)
lines(N1, sapply(1:length(N1), function(n){length(which(six_sign_5depth[n] > quants_5depth[[n]]))/1e3}),
      col = "darkgreen", type = "o", pch = 8)
lines(N2, sapply(1:length(N2), function(n){length(which(six_sign_6depth[n] > quants_6depth[[n]]))/1e3}),
      col = "brown", type = "o", pch = 10)
lines(N1, sapply(1:length(N1), function(n){length(which(six_sign_7depth[n] > quants_7depth[[n]]))/1e3}),
      col = "orange", type = "o", pch = 11)
legend("topright", legend = c("3-depth", "4-depth", "5-depth","6-depth", "7-depth"), 
       col = c("black", "red", "darkgreen","brown", "orange"), pch = c(17,15, 8,10,11))


###############################################################################################
############## SEVEN sign changes ##############################################################
##############################################################################################

generator_extra357 <- function(N){
  return(c(rep(1, N/14), rep(-1, N/7), rep(1, N/7), rep(-1, N/7), rep(1, N/7), rep(-1, N/7), rep(1, N/7), rep(-1, N/14)))
}

generator_extra468 <- function(N){
  return(c(rep(1, N/8), rep(-1, N/8), rep(1, N/8), rep(-1, N/8), rep(1, N/8), rep(-1, N/8), rep(1, N/8), rep(-1, N/8)))
}

### generate ecdfs ############################################

set.seed(2409)
N1 <- seq(14, 84, 14)
N2 <- seq(8, 88, 8)

quants_3depth <- list(replicate(1000, asymp_K_depth(rnorm(14),3)))
quants_4depth <- list(replicate(1000, asymp_K_depth(rnorm(8),4)))
quants_5depth <- list(replicate(1000, asymp_K_depth(rnorm(14),5)))
quants_6depth <- list(replicate(1000, asymp_K_depth(rnorm(8),6)))
quants_7depth <- list(replicate(1000, asymp_K_depth(rnorm(14),7)))
quants_8depth <- list(replicate(1000, asymp_K_depth(rnorm(8),8)))

for(n in N1[2:length(N1)]){
  quants_3depth <- c(quants_3depth, list(replicate(1000, asymp_K_depth(rnorm(n),3))))
  quants_5depth <- c(quants_5depth, list(replicate(1000, asymp_K_depth(rnorm(n),5))))
  quants_7depth <- c(quants_7depth, list(replicate(1000, asymp_K_depth(rnorm(n),7))))
}

for(n in N2[2:length(N2)]){
  quants_4depth <- c(quants_4depth, list(replicate(1000, asymp_K_depth(rnorm(n),4))))
  quants_6depth <- c(quants_6depth, list(replicate(1000, asymp_K_depth(rnorm(n),6))))
  quants_8depth <- c(quants_6depth, list(replicate(1000, asymp_K_depth(rnorm(n),8))))
}

#################################################################
### compute p-values

### six sign changes

six_sign_3depth <- sapply(N1, function(N){asymp_K_depth(generator_extra357(N),3)})
six_sign_4depth <- sapply(N2, function(N){asymp_K_depth(generator_extra468(N),4)})
six_sign_5depth <- sapply(N1, function(N){asymp_K_depth(generator_extra357(N),5)})
six_sign_6depth <- sapply(N2, function(N){asymp_K_depth(generator_extra468(N),6)})
six_sign_7depth <- sapply(N1, function(N){asymp_K_depth(generator_extra357(N),7)})
six_sign_8depth <- sapply(N2, function(N){asymp_K_depth(generator_extra468(N),8)})

################################################################################

# p-value plots

plot(N1, sapply(1:length(N1), function(n){length(which(six_sign_3depth[n] > quants_3depth[[n]]))/1e3}),
     type = "o", ylab = "p-value", xlab = "N", main = "Seven sign changes", pch = 17, ylim = c(0,1))
lines(N2, sapply(1:length(N2), function(n){length(which(six_sign_4depth[n] > quants_4depth[[n]]))/1e3}),
      col = "red", type = "o", pch = 15)
lines(N1, sapply(1:length(N1), function(n){length(which(six_sign_5depth[n] > quants_5depth[[n]]))/1e3}),
      col = "darkgreen", type = "o", pch = 8)
lines(N2, sapply(1:length(N2), function(n){length(which(six_sign_6depth[n] > quants_6depth[[n]]))/1e3}),
      col = "brown", type = "o", pch = 10)
lines(N1, sapply(1:length(N1), function(n){length(which(six_sign_7depth[n] > quants_7depth[[n]]))/1e3}),
      col = "orange", type = "o", pch = 11)
lines(N2, sapply(1:length(N2), function(n){length(which(six_sign_8depth[n] > quants_8depth[[n]]))/1e3}),
      col = "violet", type = "o", pch = 4)
legend("topright", legend = c("3-depth", "4-depth", "5-depth","6-depth", "7-depth", "8-depth"), 
       col = c("black", "red", "darkgreen","brown", "orange", "violet"), pch = c(17,15, 8,10,11,4))
